import tempfile
from anews.dao import msn
from anews.dao import sound
from anews.dao import create_msn_news, create_msn_img
from anews.common import utils
from gbackup import *
from zipfile import ZipFile
import uuid
import os

class anews():
    def __init__(self):
        print("okkk!")

    def open(self):
        self.root_dir = tempfile.TemporaryDirectory()

    def create_msn_news_img(self, id, url, font_url, logo_url, background_text_url, type, drive_email, langCode ):
        font_path = utils.cache_file(font_url)
        logo_path = utils.cache_file(logo_url)
        background_text_path = utils.cache_file(background_text_url)
        rs= create_msn_img(url, self.root_dir.name, font_path, background_text_path, type, langCode, logo_path)
        if rs:
            tmp=utils.upload_file(drive_email, rs['thumb'])
            rs['thumb']=f"gdrive;;{drive_email};;{tmp}"
            tmp = utils.upload_file(drive_email, rs['video'])
            rs['video'] = f"gdrive;;{drive_email};;{tmp}"
        return rs


    def create_msn_news(self, id, url, title, font_url, logo_url, drive_email, background_url, langCode, langName, country_name='USA', category='CREATOR_VIDEO_CATEGORY_NEWS', outro_url=None):
        font_path = utils.cache_file(font_url)
        logo_path = utils.cache_file(logo_url)
        background_path = utils.cache_file(background_url)
        outro_path=None
        if outro_url:
            outro_path=utils.cache_file(outro_url)
        final_vid, final_thumnail, arr_img, content, des, tags = create_msn_news(url, title, self.root_dir.name,
                                                                                 font_path, logo_path,
                                                                                 background_path, langCode, langName)
        if final_vid:
            if outro_path:
                final_vid_tmp = utils.concat_video([final_vid,outro_path], self.root_dir.name)
                final_vid=final_vid_tmp
            final_zip_path = os.path.join(self.root_dir.name, str(uuid.uuid4().hex) + "-" + str(id) + ".zip")
            arr_files = []
            config = {"title": title, "description": des, "tag": tags, "video_path": os.path.basename(final_vid),
                      "thumb_path": os.path.basename(final_thumnail), "language": langCode, "country_name": country_name, "category": category}
            arr_files.append(utils.save_file(self.root_dir.name, "config.txt", config, True))
            arr_files.append(utils.save_file(self.root_dir.name, "images.txt", ",".join(arr_img)))
            arr_files.append(utils.save_file(self.root_dir.name, "url.txt", url))
            arr_files.append(utils.save_file(self.root_dir.name, "title.txt", title))
            arr_files.append(utils.save_file(self.root_dir.name, "content.txt", content))
            arr_files.append(utils.save_file(self.root_dir.name, "des.txt", des))
            arr_files.append(utils.save_file(self.root_dir.name, "tags.txt", tags))
            arr_files.append(final_vid)
            arr_files.append(final_thumnail)
            with ZipFile(final_zip_path, 'w') as zip_tmp:
                # writing each file one by one
                for file in arr_files:
                    zip_tmp.write(file, os.path.basename(file))
            final_drive_id = utils.upload_file(drive_email, final_zip_path)
            if final_drive_id:
                return f"gdrive;;{drive_email};;{final_drive_id}"
            else:
                return None
        return None

    def close(self):
        self.root_dir.cleanup()



from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw
import textwrap
import uuid,os
import math
from w3lib.html import remove_tags
from w3lib.html import strip_html5_whitespace
from w3lib.html import remove_tags_with_content
import re


def add_logo(image_path, logo_path, tmp_path,output=None, pos="TR"):
    if output == None:
        output = os.path.join(tmp_path, uuid.uuid4().hex + "_logo_text.jpg")
    img = Image.open(image_path)
    image_size = img.size
    img_logo = Image.open(logo_path)
    img_logo_size=img_logo.size
    img.paste(img_logo, (image_size[0]-img_logo_size[0], 0),img_logo)
    rgb_im = img.convert('RGB')
    rgb_im.save(output)
    rgb_im.close()
    img.close()
    img_logo.close()
    return output

def write_text_on_logo(image_path, tmp_path, arr_letter, arr_top_index,font_file,output=None,font_size=50,text_spacing=" "):
    if output == None:
        output = os.path.join(tmp_path, uuid.uuid4().hex + "_logo_text.jpg")
    img = Image.open(image_path)
    image_size = img.size

    white = 'rgb(255, 255, 255)'
    yellow= 'rgb(255,255,0)'
    beige='rgb(245,245,220)'
    red = 'rgb(255,0,0)'
    blue='rgb(0,0,255)'
    green = 'rgb(0,255,0)'
    margin = 10
    logo_w=0

    font = ImageFont.truetype(font_file, size=font_size, encoding="unic")
    line_height = font.getsize('hg')[1]
    heightX = (line_height * 3) + 50 #max leng 3 rows
    img_box = Image.new('RGBA', (image_size[0], heightX), (0, 0, 0, 60))
    draw = ImageDraw.Draw(img_box)
    start_x=margin
    start_y =margin
    x=start_x
    y=start_y
    max_title=15
    count_letter=0
    keep_color=False
    last_color= white
    for letter in arr_letter:
        count_letter+=1
        color=white
        if(len(arr_top_index)>0 and letter.value == arr_top_index[0]):
            color=red
        if (len(arr_top_index)>1 and letter.value == arr_top_index[1]):
            color = yellow
        if (len(arr_top_index)>2 and letter.value == arr_top_index[2]):
            color = green
        if(letter.value == 1):
            color = white

        letter_x=font.getsize(letter.text+text_spacing)[0]
        if x+letter_x>image_size[0]-margin-logo_w:
            x=start_x
            y+=line_height
        if not letter.text.islower():
            if keep_color:
                color = last_color
            last_color = color
            keep_color = True
        else:
            keep_color = False
        if count_letter < max_title:
            draw.text((x,y), letter.text,fill=color, font=font)
        else:
            draw.text((x, y), "...", fill=color, font=font)
            break
        x+=letter_x
    #img.paste(img_box, (0, 0),img_box)
    heightY = heightX + 50
    img.paste(img_box, (0, image_size[1] - heightX), img_box)
    rgb_im = img.convert('RGB')
    rgb_im.save(output)
    rgb_im.close()
    img.close()
    img_box.close()
    return output




def content(image_path,text,font_file,tmp_path,wrap_text_len=70,font_size=33):
    output = os.path.join(tmp_path, uuid.uuid4().hex + "_draw_text.jpg")
    if text ==None or text =="":
        return image_path
    img = Image.open(image_path)
    image_size = img.size
    font = ImageFont.truetype(font_file, size=font_size, encoding="unic")
    lines = textwrap.wrap(text, wrap_text_len)
    line_height = font.getsize('hg')[1]
    heightX=(line_height * len(lines))+5
    # if(heightX<250):
    #     heightX=250
    img_box = Image.new('RGBA', (image_size[0], heightX), (0, 0, 0, 60))
    draw = ImageDraw.Draw(img_box)
    color = 'rgb(255, 255, 255)'
    y = 5
    x = (image_size[0] - font.getsize(lines[0])[0]) / 2
    for line in lines:
        draw.text((x, y), line, fill=color, font=font)
        # update the y position so that we can use it for next line
        y = y + line_height
    # heightY=300
    # if(heightX>heightY):
    heightY=heightX+50
    img.paste(img_box, (0, image_size[1] - heightY), img_box)
    rgb_im = img.convert('RGB')
    rgb_im.save(output)
    rgb_im.close()
    img.close()
    img_box.close()
    return output

def balance_image(arr_image,arr_text):
    if len(arr_image) < len(arr_text):
        step=1
        i=0
        while (len(arr_image) < len(arr_text)):
            if(step + i > len(arr_image)):
                i=0
                step +=1
            arr_image.insert(step+i,arr_image[i])
            i+=step+1
    if len(arr_text) < len(arr_image):
        d = len(arr_image) - len(arr_text)
        i=0
        while(i<d):
            arr_text.append("")
            i+=1
def filter_text(text,subs=None):
    rs=re.sub(r'\r\n\s+','. ',strip_html5_whitespace((remove_tags(remove_tags_with_content(text, which_ones=('sup', 'script','style'))))))
    if subs is not None and len(rs) > subs:
        rs=rs[:subs] +"..."
    return rs


def nomalize_text(arr_text,max_text_read):
        i=0
        while(i<len(arr_text)):
            arr_text[i] = filter_text(arr_text[i])
            if(arr_text[i]==None or arr_text[i]==""):
                arr_text.pop(i)
            else:
                if len(textwrap.wrap(arr_text[i], math.ceil(max_text_read*1.5)))>1 :
                    arr_tmp=textwrap.wrap(arr_text[i],math.ceil(max_text_read))
                    arr_text[i]=arr_tmp[0]
                    arr_tmp.pop(0)
                    arr_text.insert(i+1,"".join(arr_tmp))
                i+=1
def clean_text(arr_text,subs=None):
    i = 0
    while (i < len(arr_text)):
        arr_text[i] = filter_text(arr_text[i],subs)
        i+=1


import random

import requests
import os, json
import tempfile
import base64
import uuid,math,shutil
from gbackup import *
import numpy as np
from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

from anews.common.VideoEffect import create_vid_text_img
import  anews.common.FFmpeg as FFmpeg
import textwrap
from moviepy.editor import *
from slugify import slugify

def upload_file(email, path):
    drh = DriverHelper()
    conf = drh.create_driver_config(email)
    file_name = os.path.basename(path)
    return Client(conf, "upload", path, "").upload_file(file_name,path, "ROOT_FOLDER")
def download_gdrive(id, email, path):
    drh = DriverHelper()
    conf = drh.create_driver_config(email)
    Client(conf, "download", path, "").download_file(id,path)

def cache_music_bg(driveId, ext="mp3"):
    rs = None
    drid = driveId.split(";;")[-1]
    try:
        rs = os.path.join(get_dir('cached'), f"{drid}.{ext}")
        if os.path.exists(rs):
            return rs  # cached
        dh = DriverHelper()
        rs = dh.download_file_pub('live',drid, rs)
    except:
        import traceback
        traceback.print_exc()
        rs = None
        pass
    return rs
def create_arr_vid_text(title, arr_img, arr_text, root_path, bg_muic_url=None, is_delete=True):
    i=0
    arr_input = []
    while i < len(arr_text):
        arr_input.append(create_vid_text_img(arr_img[i], arr_text[i], root_path,random.randint(0,7)))
        i += 1
    final_path = concat_video(arr_input, root_path)
    if bg_muic_url:
        bg_path = cache_file(bg_muic_url)
        final_path=FFmpeg.create_video_with_audio_bg(final_path, bg_path, rs_path=os.path.join(get_dir('results'), f"{slugify(title)}-final-vid.mp4"))
    if is_delete:
        for item in arr_input:
            os.remove(item)
        for item in arr_img:
            os.remove(item)
        for item in arr_text:
            os.remove(item)
    return final_path
def make_image_hd(img_path, root_dir,direct=3,w=1280,h=720):
    out=f"{root_dir}/{str(uuid.uuid4())}.jpg"
    arr_direct=[-6,-9,-18,180,18,9,6]
    direct_rs=arr_direct[direct]
    rotate_code=f",rotate=PI/{direct_rs}:c=none:ow=rotw(PI/{direct_rs}):oh=roth(PI/{direct_rs})"
    if direct==3:
        rotate_code=""
    cmd=f'ffmpeg -i "{img_path}" -filter_complex "[0:v] split=2 [video0-1][video0-2];[video0-1] ' \
        f'scale=w={w}:h={h},boxblur=luma_radius=min(h\,w)/20:luma_power=1:chroma_radius=min(cw\,h)/20:chroma_power=1,' \
        f'setsar=1 [bg0];[video0-2] scale=w=min(iw\,min(iw*{h}/ih\,{w})):h=min(ih\,min(ih*{w}/iw\,{h})),' \
        f'setsar=1{rotate_code} [video0-2-scaled];[bg0][video0-2-scaled] overlay=x=(W-w)/2:y=(H-h)/2 [video0]" -map "[video0]" "{out}"'
    cmd_bigger = f'ffmpeg -i "{img_path}" -filter_complex "[0:v] split=2 [video0-1][video0-2];[video0-1] ' \
          f'scale=w={w}:h={h},boxblur=luma_radius=min(h\,w)/20:luma_power=1:chroma_radius=min(cw\,h)/20:chroma_power=1,' \
          f'setsar=1 [bg0];[video0-2] scale=w=min(iw*{h}/ih\,{w}):h=min(ih*{w}/iw\,{h}),' \
          f'setsar=1{rotate_code} [video0-2-scaled];[bg0][video0-2-scaled] overlay=x=(W-w)/2:y=(H-h)/2 [video0]" -map "[video0]" "{out}"'
    #print(cmd_bigger)
    os.popen(cmd_bigger).read()
    return out
def create_loop(clip_path, loop_duration, root_path):
    try:
        clip = VideoFileClip(clip_path)
        clip_duration = clip.duration
        clip.close()
        if clip_duration > loop_duration:
            return clip_path
        tmp_clip_path =f'{root_path}/{str(uuid.uuid4())}-{os.path.basename(clip_path)}'
        shutil.copyfile(clip_path, tmp_clip_path)

        times = int(math.ceil(loop_duration / clip_duration))
        file_merg_path =f'{root_path}/{str(uuid.uuid4())}'
        final_clip_path =f'{root_path}/{str(uuid.uuid4())}-final-{os.path.basename(clip_path)}'
        file_merg = open(file_merg_path, "a")
        for i in range(times):
            file_merg.write("file '%s'\n" % os.path.basename(tmp_clip_path))
        file_merg.close()
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        os.remove(tmp_clip_path)
        os.remove(file_merg_path)
        clip = VideoFileClip(final_clip_path)
        clip_duration = clip.duration
        clip.close()
        if clip_duration < loop_duration:
            return None
        return final_clip_path
    except:
        return None
def concat_video(inputs, root_path):
    try:
        arr_tmp=[]
        file_merg_path = f'{root_path}/{str(uuid.uuid4())}'
        file_merg = open(file_merg_path, "a")
        for input in inputs:
            tmp_clip_path = f'{root_path}/{str(uuid.uuid4())}-{os.path.basename(input)}'
            arr_tmp.append(tmp_clip_path)
            shutil.copyfile(input, tmp_clip_path)
            file_merg.write("file '%s'\n" % os.path.basename(tmp_clip_path))
        final_clip_path = f'{root_path}/{str(uuid.uuid4())}-final-{os.path.basename(input)}'
        file_merg.close()
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        for tmp_p in arr_tmp:
            os.remove(tmp_p)
        os.remove(file_merg_path)
        return final_clip_path
    except:
        return None
def download_file(url, root_dir=None, ext= None):
    rs = None
    try:
        if ext:
            file_name = str(uuid.uuid4()) + "." + ext
        else:
            file_name = os.path.basename(url)
        if not root_dir:
            rs = os.path.join(get_dir('download'),file_name)
        else:
            rs = os.path.join(root_dir, file_name)
        if "gdrive" in url:
            download_gdrive(url.split(";;")[-1],url.split(";;")[-2],rs)
        else:
            r = requests.get(url)
            with open(rs, 'wb') as f:
                f.write(r.content)
    except:
        rs = None
        pass
    return rs

def cache_file(url):
    if not url:
        return None
    rs = None
    try:
        rs = os.path.join(get_dir('cached'),os.path.basename(url))
        if os.path.exists(rs):
            return rs #cached
        r = requests.get(url)
        with open(rs, 'wb') as f:
            f.write(r.content)
    except:
        rs = None
        pass
    return rs


def save_file(root,name,content,is_json=False):
    path=os.path.join(root,name)
    with open(path,"w", encoding='utf-8') as f:
        if is_json:
            json.dump(content, f)
        else:
            f.write(content)
    return path

def get_dir(dir):
    tmp_download_path = os.path.join(tempfile.gettempdir() ,dir)
    if not os.path.exists(tmp_download_path):
        os.makedirs(tmp_download_path)
    return tmp_download_path

def calsizeOfCharacter(font, text):
    avg_char_width = sum(font.getsize(char)[0] for char in text) / len(text)
    return int(avg_char_width)
def draw_text_img(text, font_path, bg_path, root_path, margin=50):
    final_path = f"{root_path}/txt-on-img-{str(uuid.uuid4())}.png"
    fontsize = 33
    max_char_count = 90
    font = ImageFont.truetype(font_path, fontsize)
    img = Image.open(bg_path)
    w , h= img.size
    draw = ImageDraw.Draw(img)
    while True:
        scaled_wrapped_text = textwrap.fill(text=text, width=max_char_count)
        x = draw.textsize(scaled_wrapped_text, font)
        if x[1] > h:
            fontsize -= 1
            font = ImageFont.truetype(font_path, fontsize)
            max_char_count = 90
            continue
        if x[0] > w-2*margin:
            bias = max(int((x[0] - (w-2*margin)) / calsizeOfCharacter(font, text)), 1)
            max_char_count -= bias
            continue
        break
    draw.text((margin, 0), scaled_wrapped_text, (255, 255, 255), font=font)
    img.save(final_path)
    return final_path


from moviepy.editor import VideoFileClip, AudioFileClip
import anews.common.utils as utils
import uuid, math
import os
import shutil

def create_loop_audio_times(audio_path, times, is_del=True):
    try:
        tmp_clip_path = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(audio_path))
        shutil.copyfile(audio_path, tmp_clip_path)
        file_merg_path = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
        final_clip_path = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final-' + os.path.basename(audio_path))
        file_merg = open(file_merg_path, "a")
        for i in range(times):
            file_merg.write("file '%s'\n" % tmp_clip_path)
        file_merg.close()
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        os.remove(tmp_clip_path)
        os.remove(file_merg_path)
        if is_del:
            os.remove(audio_path)
        return final_clip_path
    except:
        pass
    return None
def create_loop_audio(audio_path, loop_duration):
    if loop_duration < 0 : loop_duration = 600
    try:
        audio_clip = AudioFileClip(audio_path)
        clip_duration = audio_clip.duration
        audio_clip.close()
        if clip_duration > loop_duration:
            return audio_path
        tmp_clip_path = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(audio_path))
        shutil.copyfile(audio_path, tmp_clip_path)
        times = int(math.ceil(loop_duration / clip_duration))
        file_merg_path = os.path.join(utils.get_dir('coolbg_ffmpeg') , str(uuid.uuid4()))
        final_clip_path = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final-' + os.path.basename(audio_path))
        file_merg = open(file_merg_path, "a")
        for i in range(times):
            file_merg.write("file '%s'\n" % tmp_clip_path)
        file_merg.close()
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        os.remove(tmp_clip_path)
        os.remove(file_merg_path)
        clip = AudioFileClip(final_clip_path)
        clip_duration = clip.duration
        clip.close()
        if clip_duration < loop_duration:
            return None
        return final_clip_path
    except:
        pass
    return None
def add_null_sound(input):
    output=  os.path.join(utils.get_dir('coolbg_ffmpeg') ,str(uuid.uuid4()) + '-' + os.path.basename(input))
    cmd = f"ffmpeg -f lavfi -i anullsrc -i \"{input}\" -c:v copy -b:a 128000 -ar 44100 -c:a mp3 -map 0:a -map 1:v -shortest \"{output}\""
    os.system(cmd)
    return output

def merge_list_video(vids, is_del=True):
    file_merg_path = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
    final_clip_path = os.path.join(utils.get_dir('results') , str(uuid.uuid4()) + '-final-' + os.path.basename(vids[0]))
    file_merg = open(file_merg_path, "a")
    arrtmp = []
    for vid in vids:
        tmp_clip_path = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(vid))
        shutil.copyfile(vid, tmp_clip_path)
        if is_del:
            os.remove(vid)
        file_merg.write("file '%s'\n" % tmp_clip_path)
        arrtmp.append(tmp_clip_path)
    file_merg.close()
    cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
    os.system(cmd)
    os.remove(file_merg_path)
    for item in arrtmp:
        os.remove(item)
    clip = VideoFileClip(final_clip_path)
    clip_duration = clip.duration
    clip.close()
    if clip_duration < 1:
        return None
    return final_clip_path


def merge_intro_outro(clip_path, intro = None, outro = None):
    arrVids=[]
    if not intro and not outro:
        return clip_path
    if intro:
        arrVids.append(intro)
    arrVids.append(clip_path)
    if outro:
        arrVids.append(outro)
    return merge_list_video(arrVids, True)

def create_video_audio(clip_path, audio_path, rs_path=None, is_del=True):
    try:
        tmp_clip_path_z=rs_path
        if tmp_clip_path_z is None:
            tmp_clip_path_z = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final.mp4')
        cmd = f"ffmpeg -i \"{clip_path}\" -i \"{audio_path}\" -vcodec copy -acodec copy -map 0:v -map 1:a -shortest -flags global_header -y \"{tmp_clip_path_z}\""
        os.system(cmd)
        if is_del:
            os.remove(clip_path)
            os.remove(audio_path)
    except:
        tmp_clip_path_z = None
        pass
    return tmp_clip_path_z
def create_video_with_audio_bg(clip_path, audio_path, rs_path=None, is_del=True):
    try:
        tmp_clip_path_z=rs_path
        if tmp_clip_path_z is None:
            tmp_clip_path_z = os.path.join(utils.get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final.mp4')
        vid_clip = VideoFileClip(clip_path)
        clip_duration = vid_clip.duration
        vid_clip.close()
        audio_path=create_loop_audio(audio_path, clip_duration)
        cmd = f"ffmpeg -i \"{clip_path}\" -i \"{audio_path}\" -vcodec copy -acodec copy -map 0:v -map 1:a -shortest -flags global_header -y \"{tmp_clip_path_z}\""
        os.system(cmd)
        if is_del:
            os.remove(clip_path)
            if "cache" not in audio_path:
                os.remove(audio_path)
    except:
        tmp_clip_path_z = None
        pass
    return tmp_clip_path_z



from .sound import get_voice
from .msn import crawl_data, crawl_data_slideshow
from anews.common.utils import make_image_hd,create_loop, draw_text_img, create_arr_vid_text
import textwrap
from moviepy.editor import *
import math,uuid,random
import traceback
from .texthelper import make_title,make_tag

def  balance_img_txt(arr_text, arr_img):
    if len(arr_img) < len(arr_text):
        step=1
        i=0
        while len(arr_img) < len(arr_text):
            if step + i > len(arr_img):
                i=0
                step += 1
            arr_img.insert(step + i, arr_img[i]);
            i += 1 + step
    while len(arr_text) < len(arr_img):
        arr_text.append("...")
    return arr_text, arr_img
def create_msn_img(url, root_path, font_path, bg_path, bg_muic_url, type="slideshow", langCode='en',logo_path=None, atleast_sentense=3):
    if type=="slideshow":
        arr_text, arr_img, title = crawl_data_slideshow(url)
    else:
        arr_text, arr_img, title = crawl_data(url)

    if arr_text is None or len(arr_text) == 0 or len(arr_img) == 0 or len(arr_text) < atleast_sentense:
        print("None data")
        return None
    arr_text, arr_img = balance_img_txt(arr_text, arr_img)
    arr_text_img=[]
    arr_img_hd=[]
    full_text = "".join(arr_text)
    full_text = full_text.strip()
    for text in arr_text:
        arr_text_img.append(draw_text_img(text, font_path, bg_path, root_path))
    for img in arr_img:
        arr_img_hd.append(make_image_hd(img, root_path))
    thumbnail_img = random.choice(arr_img)
    thumbnail_ok = make_title(title, full_text, make_image_hd(thumbnail_img, root_path),
                              font_path, 88, langCode, root_path, logo_path)
    tags = make_tag(title, full_text, langCode)
    tags = textwrap.wrap(tags, 500)[0]
    final_path = create_arr_vid_text(title, arr_img_hd, arr_text_img, root_path, bg_muic_url)
    rs = {"arr_text": arr_text, "arr_img": arr_img, "tags": tags, "thumb": thumbnail_ok, "video": final_path}
    print(final_path)
    return rs

def create_msn_news(url, title, root_path, font_path, logo_path,background_path, langCode, langName):
    arr_text, arr_img= crawl_data(url)
    if arr_text is None or len(arr_text)==0 or len(arr_img)==0:
        print("None data")
        return None, None, None, None, None,None
    final_path=None
    try:
        full_text= "".join(arr_text)
        full_text = full_text.strip()
        if len(full_text) < 500:
            print("Error full_text length")
            return None, None, None, None, None, None
        thumbnail_img = random.choice(arr_img)
        thumbnail_ok = make_title(title, full_text, make_image_hd(thumbnail_img, root_path),
                   font_path, 88, langCode, root_path, logo_path)
        tags = make_tag(title,full_text, langCode)
        tags = textwrap.wrap(tags, 500)[0]
        wrapper = textwrap.TextWrapper(width=5000)
        sumary_des=textwrap.shorten(text=full_text, width=300)

        arr_wrap_text=wrapper.wrap(text=full_text)
        arr_voice=[]
        for wrap_text in arr_wrap_text:
            path_voice = get_voice(wrap_text, root_path, langCode, langName)
            arr_voice.append(AudioFileClip(path_voice))
        full_audio = concatenate_audioclips(arr_voice)
        full_audio_duration=full_audio.duration

        time_per_img=math.ceil(full_audio_duration/len(arr_img))
        arr_img_comp=[]
        bg_path=create_loop(background_path,full_audio_duration,root_path)
        bg = VideoFileClip(bg_path)
        arr_img_comp.append(bg)
        start_t=0
        TIME_SUB_IMG=15
        pos_img = (14,14)
        if "tam1" in background_path or "dung1" in background_path:
            pos_img = (18,18)
        for img in arr_img:
            arr_img_comp.append(
                ImageClip(make_image_hd(img, root_path)).set_start(start_t).set_duration(TIME_SUB_IMG).set_position(pos_img))
            start_t+=TIME_SUB_IMG
            times_sub = math.floor(time_per_img-TIME_SUB_IMG/TIME_SUB_IMG)
            total_time_sub_tmp=0
            for i in range(times_sub):
                time_duration_tmp = TIME_SUB_IMG
                if i == times_sub-1:
                    time_duration_tmp=time_per_img-total_time_sub_tmp
                arr_img_comp.append(
                    ImageClip(make_image_hd(img, root_path,i%7)).set_start(start_t).set_duration(time_duration_tmp).set_position(pos_img))
                total_time_sub_tmp+= time_duration_tmp
                start_t += time_duration_tmp
        # i=0
        # while start_t < full_audio_duration:
        #     arr_img_comp.append(arr_img_comp_tmp[i].set_start(start_t))
        #     start_t += time_per_img
        #     i = (i+1) % len(arr_img)
        final_path=f"{root_path}/final-up-{str(uuid.uuid4())}.avi"
        CompositeVideoClip(arr_img_comp).set_audio(full_audio).subclip(0, full_audio_duration).write_videofile(final_path, fps=24, codec='libx264')
        for item in arr_img_comp:
            item.close()
    except:
        print(traceback.format_exc())
        raise Exception(traceback.format_exc())
    return final_path, thumbnail_ok, arr_img, full_text, sumary_des, tags











import setuptools

setuptools.setup(
    name='anews',
    version='1.27',
    author="Thanh Hoa",
    author_email="getmoneykhmt3@gmail.com",
    description="A Des of anews",
    long_description="anews",
    long_description_content_type="text/markdown",
    url="https://github.com/vtandroid/anews",
    packages=setuptools.find_packages(),
    py_modules=['anews'],
    install_requires=[
        'pillow','requests','gbak','w3lib','python-slugify'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
 )